#include "i0mapmon.h"
#include "ui_i0mapmon.h"

// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
// cumbia

#include <quwatcher.h>
#include <qulineedit.h>
#include <qustring.h>
#include <quwriter.h>

#include "checkablecomboboxmodel.h"

I0mapmon::I0mapmon(CumbiaPool *cumbia_pool, QWidget *parent) :
    QWidget(parent)
{
    // cumbia
    CuModuleLoader mloader(cumbia_pool, &m_ctrl_factory_pool, &m_log_impl);
    cu_pool = cumbia_pool;

    ui = new Ui::I0mapmon;
    ui->setupUi(this, cu_pool, m_ctrl_factory_pool);

    // mloader.modules() to get the list of loaded modules
    // cumbia

    m_fel0x = 0;
    for(int i = 1; i < qApp->arguments().size() && m_fel0x < 1; i++) {
        if(qApp->arguments()[i].contains("f01")) m_fel0x = 1;
        else if(qApp->arguments()[i].contains("f02")) m_fel0x = 2;
    }
    if(m_fel0x < 1)
        ui->lTitle->setText("Unknown operation mode FEL01 / FEL02 from command line args");
    else {
        ui->lTitle->setText(ui->lTitle->text() + QString(" FEL0%1").arg(m_fel0x));

        m_cb_model = new CheckableComboBoxModel(ui->qucby);
        connect(m_cb_model, SIGNAL(itemChecked(QString,bool)), this, SLOT(onWlenSelectionChanged(QString, bool)));
        ui->qucblambda->setEditable(true);
        ui->qucblambda->setModel(m_cb_model);

        // continue setup
        QuWatcher *w = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        w->setSingleShot(true);
        w->setAutoDestroy(true);
        w->attach(this, SLOT(onNamesReady(QStringList)));
        w->setSource("$1/name");

        connect(ui->pbApplyWlenWei, SIGNAL(clicked()), this, SLOT(applyWavelenX()));
        connect(ui->pbApplyWlens, SIGNAL(clicked()), this, SLOT(applyWavelenX()));

        // qu-line edit for wavelen / wavelen weights
        ui->quleWlens->setTarget(QString("$2/FEL0%1_IOM_wavelengths").arg(m_fel0x));
        ui->quleWlenWei->setTarget(QString("$2/FEL0%1_IOM_wavelength_weights").arg(m_fel0x));

        // label
        ui->qulWlens->setSource(QString("$2/FEL0%1_IOM_wavelengths").arg(m_fel0x));
        ui->qulWlenWei->setSource(QString("$2/FEL0%1_IOM_wavelength_weights").arg(m_fel0x));
        connect(ui->qulWlens, SIGNAL(newData(CuData)), this, SLOT(onWlensChanged(CuData)));

        // plot
        ui->plot->setYLowerBound(-1.0);
        ui->plot->setYUpperBound(1.0);
        ui->plot->setXLowerBound(-1.0);
        ui->plot->setXUpperBound(1.0);
        // autoscale
        ui->plot->setYAxisAutoscaleEnabled(true);

        m_setup_x_list();

        // trigger plot reconf
        connect(ui->pbPlotReconf, SIGNAL(clicked()), this, SLOT(m_plot_reconf()));

        ui->gbConfPlot->setVisible(false);
        connect(ui->pbConfPlot, SIGNAL(toggled(bool)), ui->gbConfPlot, SLOT(setChecked(bool)));
        connect(ui->pbConfPlot, SIGNAL(toggled(bool)), ui->gbConfPlot, SLOT(setVisible(bool)));
        connect(ui->gbConfPlot, SIGNAL(toggled(bool)), ui->pbConfPlot, SLOT(setChecked(bool)));
        connect(ui->cbYAuto, SIGNAL(toggled(bool)), ui->leyMin, SLOT(setDisabled(bool)));
        connect(ui->cbYAuto, SIGNAL(toggled(bool)), ui->leyMax, SLOT(setDisabled(bool)));
        connect(ui->pbApplyBounds, SIGNAL(clicked()), this, SLOT(setYMinMax()));
        connect(ui->cbYAuto, SIGNAL(toggled(bool)), ui->plot, SLOT(setYBoundsAuto(bool)));
        ui->cbYAuto->setChecked(true); // disable y bounds line edits
    }
}

I0mapmon::~I0mapmon()
{
    delete ui;
}

void I0mapmon::onNamesReady(const QStringList &n) {
    qDebug() << __PRETTY_FUNCTION__ << n;
    ui->qucby->clear();
    ui->qucby->insertItems(0, n);
    if(n.size()) {
        //        onYChanged(n.first());
        // X combo box
        connect(ui->qucby, SIGNAL(currentTextChanged(QString)), this, SLOT(onYChanged(QString)));
    }
}

void I0mapmon::applyWavelenX() {
    bool ok;
    QString tgt, txt;
    QVector<double> vals;
    sender()->objectName() == "pbApplyWlens" ? txt = ui->quleWlens->text() : txt = ui->quleWlenWei->text();
    foreach(const QString& v, txt.split(QRegularExpression(",\\s*")))
        if(v.toDouble(&ok) && ok) vals.push_back(v.toDouble());
    sender()->objectName() == "pbApplyWlens" ? tgt = QString("$2/FEL0%1_IOM_wavelengths").arg(m_fel0x) :
            tgt = QString("$2/FEL0%1_IOM_wavelength_weights").arg(m_fel0x);
    if(vals.size() > 0) {
        QuWriter *w = new QuWriter(this, cu_pool, m_ctrl_factory_pool);
        w->setAutoDestroy(true);
        w->setTarget(tgt);
        w->execute(vals);
        qDebug() << __PRETTY_FUNCTION__ << "*WRITING ON* target" << w->target() << "values" << vals;
    }
}

void I0mapmon::onYChanged(const QString &y) {

}

void I0mapmon::onWlensChanged(const CuData &da) {
    ui->qucblambda->setEnabled(!da["err"].toBool());
    if(ui->qucblambda->isEnabled()) {
        m_cb_model->clear();
        m_cb_model->addItem("select...", false);
        foreach(const QString& wl, QuString(da["value"].toString()).split(",")) {
            m_cb_model->addItem(wl);
        }
    }
    ui->qucblambda->setCurrentIndex(0);
}

void I0mapmon::onWlenSelectionChanged(const QString& , bool ) {
    QStringList wlens;
    for(int i = 0; i < m_cb_model->rowCount(); i++) {
        QStandardItem *it = m_cb_model->item(i);
        if(it->data(Qt::CheckStateRole).toBool())
            wlens.append(it->text());
    }
    ui->labelleWlens->setText(wlens.join(","));
    ui->qucblambda->setItemText(0, ui->labelleWlens->text());
}

void I0mapmon::setYMinMax() {
    ui->plot->setYBounds(ui->leyMin->text().toDouble(), ui->leyMax->text().toDouble());
}

void I0mapmon::m_setup_x_list() {
    ui->qucbx->insertItem(0, "time");
    ui->qucbx->setItemData(0, "time", Qt::UserRole + 1);

    // "PM2a Ni Photocurrent (a.u.)" /pos/diagnostics/photo_current_pos.01    None    GetArea3
    // "PM2a Ca Photocurrent (a.u.)" pos/diagnostics/photo_current_pos.01    None    GetArea2
    // "PM2a OLD Photocurrent" srv-tf-srf:20000/ec-tf-esa-01/v1720/1000000     None GetArea4
    // "PRESTO Intensity"      pos/pesp/ccd-pesp_pos.01  Intensity      None
    // "TARDI Intensity"   pos_mag/pesp/ccd-pesp_pos_mag.01  Intensity      None
    //  "DPI Photodiode (Beamline Transmission)" pos/diagnostics/pico_pos.01    None    GetIc
    // "LDM Photodiode (Beamline Transmission)" pos/diagnostics/pico_pos.01    None    GetId

    ui->qucbx->insertItem(ui->qucbx->count(), "PM2a Ni Photocurrent (a.u.)");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "pos/diagnostics/photo_current_pos.01->GetArea3(0,%1)", Qt::UserRole + 1);
    ui->qucbx->insertItem(ui->qucbx->count(), "PM2a Ca Photocurrent (a.u.)");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "pos/diagnostics/photo_current_pos.01->GetArea2(0,%1)", Qt::UserRole + 1);
    ui->qucbx->insertItem(ui->qucbx->count(), "PM2a OLD Photocurrent");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "srv-tf-srf:20000/ec-tf-esa-01/v1720/1000000->GetArea4(0,%1)", Qt::UserRole + 1);
    ui->qucbx->insertItem(ui->qucbx->count(), "PRESTO Intensity");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "pos/pesp/ccd-pesp_pos.01/Intensity", Qt::UserRole + 1);
    ui->qucbx->insertItem(ui->qucbx->count(), "DPI Photodiode (Beamline Transmission)");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "pos/diagnostics/pico_pos.01->GetIc(0,%1)", Qt::UserRole + 1);
    ui->qucbx->insertItem(ui->qucbx->count(), "LDM Photodiode (Beamline Transmission)");
    ui->qucbx->setItemData(ui->qucbx->count() - 1, "pos/diagnostics/pico_pos.01->GetId(0,%1)", Qt::UserRole + 1);
}

QMap<int, double> I0mapmon::m_selected_wlens() const {
    QMap<int, double> m;
    for(int i = 1; i < m_cb_model->rowCount(); i++) {// item 0: "time"
        qDebug() << __PRETTY_FUNCTION__ << "model row count" << m_cb_model->rowCount() << "row" << i << "checked" << m_cb_model->item(i)->data(Qt::CheckStateRole).toBool()
                 << "text " << m_cb_model->item(i)->data(Qt::DisplayRole).toString() << ui->qucblambda->itemText(i).toDouble();
        if(m_cb_model->item(i)->data(Qt::CheckStateRole).toBool())  {
            m[i - 1] = ui->qucblambda->itemText(i).toDouble();
        }
    }
    return m;
}

void I0mapmon::m_plot_reconf() {
    const QString& srcx = ui->qucbx->itemData(ui->qucbx->currentIndex(), Qt::UserRole + 1).toString();
    const QString devy = ui->quionSamples->source().remove("/" + ui->quionSamples->source().section('/',- 1));
    printf("\e[1;32mI0mapmon.m_plot_reconf x \"%s\" y \"%s\"\e[0m\n", qstoc(srcx), qstoc(devy));
    ui->plot->setSources(srcx, devy + '/' + ui->qucby->currentText(), m_selected_wlens(), ui->quionSamples->findChild<QuLabel *>()->text().toInt());
}
